using System;
using System.Collections.Generic;
using System.Text;

namespace Gdal
{
    /// <summary>
    /// The coefficients for transforming between pixel/line (X,Y) raster space, and projection coordinates (Xp,Yp) space.<br/>
    /// Xp = T[0] + T[1]*X + T[2]*Y<br/>
    /// Yp = T[3] + T[4]*X + T[5]*Y<br/>
    /// In a north up image, T[1] is the pixel width, and T[5] is the pixel height.
    /// The upper left corner of the upper left pixel is at position (T[0],T[3]).
    /// </summary>
    public class GeoTransform
    {
        internal double[] transform = new double[6];

        public GeoTransform()
        {
            transform = new double[6];
            transform[0] = 0;
            transform[1] = 1;
            transform[2] = 0;
            transform[3] = 0;
            transform[4] = 0;
            transform[5] = 1;
        }

        public GeoTransform(double[] array)
        {
            if (array.Length != 6)
                throw new ApplicationException("GeoTransform constructor invoked with invalid sized array");
            transform = array;
        }

        double ProjectedX(double imgX, double imgY)
        {
            return transform[0] + transform[1] * imgX + transform[2] * imgY;
        }

        double ProjectedY(double imgX, double imgY)
        {
            return transform[3] + transform[4] * imgX + transform[5] * imgY;
        }

        public double this[int i]
        {
            get
            {
                return transform[i];
            }
            set
            {
                transform[i] = value;
            }
        }

        bool IsTrivial
        {
            get
            {
                return transform[0] == 0 && transform[1] == 1 &&
                    transform[2] == 0 && transform[3] == 0 &&
                    transform[4] == 0 && transform[5] == 1;
            }
        }

    }
}
