using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace Gdal
{

    internal enum CPLErr
    {
        None = 0,
        Debug = 1,
        Warning = 2,
        Failure = 3,
        Fatal = 4
    }


    /// <summary>
    /// Common location for all GDAL InterOp method declarations
    /// </summary>
    internal class InterOp
    {

        static InterOp()
        {
            GDALAllRegister();
        }

        const string gdalDll = "gdal_fw.dll";

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALOpen([MarshalAs(UnmanagedType.LPStr)] string pszFilename, Access eAccess);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDatasetDriver(HandleRef hDataset);


        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterXSize(HandleRef hDataset);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterYSize(HandleRef hDataset);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterCount(HandleRef hDataset);

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetProjectionRef")]
        private static extern IntPtr GDALGetProjectionRefExt(HandleRef hDataset);

        internal static string GDALGetProjectionRef(HandleRef hDataset)
        {
            return Marshal.PtrToStringAnsi(GDALGetProjectionRefExt(hDataset));
        }

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALGetGeoTransform(HandleRef hDataset, [In, Out] double[] Transform);

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALSetGeoTransform(HandleRef hDataset, [In, Out] double[] Transform);


        [DllImport(gdalDll)]
        internal static extern void GDALClose(HandleRef hDriver);

        [DllImport(gdalDll, EntryPoint = "CPLGetLastErrorMsg")]
        private static extern IntPtr CPLGetLastErrorMsgExt();

        internal static string CPLGetLastErrorMsg()
        {
            return Marshal.PtrToStringAnsi(CPLGetLastErrorMsgExt());
        }

        [DllImport(gdalDll)]
        internal static extern void GDALAllRegister();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALCreateCopy(HandleRef hDriver, [MarshalAs(UnmanagedType.LPStr)] string filename,
            HandleRef hDataset, int bStrict, HandleRef options, HandleRef progressFn, HandleRef progressData);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALCreate(HandleRef hDriver, [MarshalAs(UnmanagedType.LPStr)] string filename,
            int width, int height, int nBands, DataType type, HandleRef options);

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDriverByName([MarshalAs(UnmanagedType.LPStr)]string pszName);


        [DllImport(gdalDll)]
        internal static extern int GDALGetDriverCount();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetDriver(int iDriver);

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDriverShortName")]
        private static extern IntPtr GDALGetDatasetDriverExt(HandleRef hDataset);

        internal static string GDALGetDriverShortName(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDatasetDriverExt(hDriver));
        }

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDriverLongName")]
        private static extern IntPtr GDALGetDriverLongNameExt(HandleRef hDriver);

        internal static string GDALGetDriverLongName(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDriverLongNameExt(hDriver));
        }

        [DllImport(gdalDll, CharSet = CharSet.Ansi, EntryPoint = "GDALGetDescription")]
        private static extern IntPtr GDALGetDescriptionExt(HandleRef hDriver);

        internal static string GDALGetDescription(HandleRef hDriver)
        {
            return Marshal.PtrToStringAnsi(GDALGetDescriptionExt(hDriver));
        }

        [DllImport(gdalDll)]
        internal static extern void GDALDestroyDriverManager();

        [DllImport(gdalDll)]
        internal static extern IntPtr GDALGetRasterBand(HandleRef hDataset, int index);

        [DllImport(gdalDll)]
        internal static extern CPLErr GDALRasterIO(HandleRef hBand, RWFlag flag, int xOff, int yOff,
            int xSize, int ySize, IntPtr data, int bufXSize, int bufYSize, DataType type, int pixelStride,
            int lineStride);

        [DllImport(gdalDll)]
        internal static extern int GDALGetRasterColorInterpretation(HandleRef hBand);

        [DllImport(gdalDll)]
        internal static extern int GDALGetAccess(HandleRef hDataset);

        [DllImport(gdalDll, EntryPoint="GDALGetMetadata")]
        private static extern IntPtr GDALGetMetadataExt(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string domain);

        internal unsafe static Dictionary<string, string> GDALGetMetadata(HandleRef hDataset, string domain)
        {
            Dictionary<string, string> data = new Dictionary<string, string>();
            void** stringList = (void**) GDALGetMetadataExt(hDataset, domain);
            if (new IntPtr(stringList) != IntPtr.Zero) {
                while (new IntPtr(*stringList) != IntPtr.Zero) {
                    string keyValue = Marshal.PtrToStringAnsi(new IntPtr(*stringList));
                    string[] elements = keyValue.Split("=".ToCharArray());
                    if (elements.Length < 2)
                        throw new ApplicationException("Parse error while parsing string list in GDALGetMetadata()");
                    data.Add(elements[0], elements[1]);
                    ++stringList;
                }
            }
            return data;
        }

        [DllImport(gdalDll, EntryPoint="GDALGetMetadataItem")]
        private static extern IntPtr GDALGetMetadataItemExt(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string key, [MarshalAs(UnmanagedType.LPStr)] string domain);

        internal static string GDALGetMetadataItem(HandleRef hDataset, string key, string domain)
        {
            return Marshal.PtrToStringAnsi(GDALGetMetadataItemExt(hDataset, key, domain));
        }

        [DllImport(gdalDll)]
        internal static extern void GDALSetMetadataItem(HandleRef hDataset,
            [MarshalAs(UnmanagedType.LPStr)] string key, [MarshalAs(UnmanagedType.LPStr)] string value,
            [MarshalAs(UnmanagedType.LPStr)] string domain);
    }
}
